#ifdef __WITH_SDL__
#include "SdlHelper.h"
#else
#include "BmpHelper.h"
#endif
#include <time.h>
#include <iostream>
#include <iomanip>
#include <stdexcept>
#include "../../include/CoaxlinkGrabber.h"

using namespace Euresys;

void sample() {
    const size_t FRAME_COUNT = 20;
    const uint64_t POP_TIMEOUT = 1000;

    BmpHelper::Init8bppHeaders();

    Euresys::GenTL genTL;
    CoaxlinkGrabber grabber(genTL);
    grabber.runScript("./config.js");
    grabber.ReallocBuffers(FRAME_COUNT);

    const int64_t width = grabber.GetWidth();
    const int64_t height = grabber.GetHeight();

    grabber.Start(FRAME_COUNT);
    for (unsigned frame = 0; frame < FRAME_COUNT; ++frame) {
        CoaxlinkGrabber::ScopedBuffer buffer(grabber, POP_TIMEOUT);
        size_t size = buffer.getInfo<size_t>(gc::BUFFER_INFO_SIZE_FILLED);
        uint64_t timestamp = buffer.getInfo<uint64_t>(gc::BUFFER_INFO_TIMESTAMP);
        unsigned char *imgPtr = buffer.getInfo<unsigned char *>(gc::BUFFER_INFO_BASE);

        std::cout << "Acquisition Index: " << frame
                  << " - Width: " << width
                  << " - Height: " << height
                  << " - Size: " << size
                  << " - Timestamp: " << std::setw(0) << (timestamp / 1000000) << "." 
                  << std::setw(6) << std::setfill('0') << (timestamp % 1000000)
                  << std::endl;

        if ((int) size != width * height) {
            std::cout << " - skip saving: not in Mono8 format" << std::endl;
        } else {
            std::stringstream ss;
            ss << "frame_" << frame << ".bmp";
            std::string fname(ss.str());
            std::cout << " + saving to " << fname << std::endl;
            BmpHelper::SaveTo8bppBmpFile(fname.c_str(), (LONG)width, (LONG)height, imgPtr);
        }
    }
}

int main() {
    try {
        sample();
    } catch (const std::exception &e) {
        std::cerr << e.what() << std::endl;
        return 1;
    } catch (...) {
        std::cerr << "uncaught exception!" << std::endl;        
        return 1;
    }
    return 0;
}
